"""World Weather Clock."""
from pytz import timezone
import requests
import sys
from time import sleep
from datetime import datetime
from Adafruit_CharLCD import Adafruit_CharLCD
import Adafruit_DHT as dht

lcd = Adafruit_CharLCD(rs=26, en=19, d4=13, d5=6, d6=5, d7=11,
                       cols=16, lines=2)
lcd.clear()

# Yahoo Weather City ID's more available at https://weather.yahoo.com
cities = [('Amsterdam', '52.37', '4.90', 'Europe/Amsterdam'),
          ('Athens', '37.98', '23.73', 'Europe/Athens'),
          ('Bangkok',  '13.76', '100.50', 'Asia/Bangkok'),
          ('Beijing', '39.90', '116.41', 'Asia/Shanghai'),
          ('Berlin', '52.52', '13.41', 'Europe/Berlin'),
          ('Bern', '46.95', '7.45', 'Europe/Zurich'),
          ('Brussels', '50.85', '4.35', 'Europe/Brussels'),
          ('Bucharest', '44.42', '26.10', 'Europe/Bucharest'),
          ('Budapest', '47.50', '19.04', 'Europe/Budapest'),
          ('Cape Town', '-33.93', '18.42', 'Africa/Johannesburg'),
          ('Chicago', '41.88', '-87.63', 'America/Chicago'),
          ('Dhaka', '23.81', '90.41', 'Asia/Dhaka'),
          ('Dublin', '53.34', '-6.26', 'Europe/Dublin'),
          ('Hanoi', '21.03', '105.83', 'Asia/Jakarta'),
          ('Havana', '23.09', '-82.36', 'America/Havana'),
          ('Helsinki', '60.17', '24.94', 'Europe/Helsinki'),
          ('Hollywood', '34.09', '-118.31', 'America/Los_Angeles'),
          ('Istanbul', '41.01', '28.98', 'Europe/Istanbul'),
          ('Jaipur', '26.93', '75.86', 'Asia/Calcutta'),
          ('Jakarta', '-6.21', '106.85', 'Asia/Jakarta'),
          ('Jerusalem', '31.77', '35.21', 'Asia/Jerusalem'),
          ('Kiev', '50.44', '30.51', 'Europe/Kiev'),
          ('Kuantan', '3.77', '103.26', 'Asia/Kuala_Lumpur'),
          ('Lima', '-12.04', '-77.04', 'America/Lima'),
          ('Lisbon', '38.73', '-9.14', 'Europe/Lisbon'),
          ('Ljubljana', '46.06', '14.51', 'Europe/Ljubljana'),
          ('London',   '51.51', '-0.13', 'Europe/London'),
          ('Madrid', '40.41', '-3.70', 'Europe/Madrid'),
          ('Manila', '14.60', '120.98', 'Asia/Manila'),
          ('Marrakesh', '31.65', '-7.97', 'Africa/Casablanca'),
          ('McMurdo', '-77.84', '166.68', 'Antarctica/McMurdo'),
          ('Melbourne', '-37.81',  '144.96', 'Australia/Melbourne'),
          ('Montreal', '45.50', '-73.57', 'America/Montreal'),
          ('Moscow', '55.76', '37.59', 'Europe/Moscow'),
          ('Mumbai', '19.09', '72.88', 'Asia/Calcutta'),
          ('New York', '40.71', '-74.01', 'America/New_York'),
          ('Oaxaca', '17.07', '-96.73', 'America/Mexico_City'),
          ('Oslo', '59.91', '10.75', 'Europe/Oslo'),
          ('Paris', '48.85', '2.35', 'Europe/Paris'),
          ('Prague', '50.08', '14.42', 'Europe/Prague'),
          ('Rio', '-22.91', '-43.17', 'America/Sao_Paulo'),
          ('Rome', '41.89', '12.48', 'Europe/Rome'),
          ('Santiago', '-33.45', '-70.68', 'America/Santiago'),
          ('Seoul', '37.57', '126.98', 'Asia/Seoul'),
          ('Singapore', '1.35', '103.85', 'Asia/Singapore'),
          ('Sofia', '42.70', '23.32', 'Europe/Sofia'),
          ('Stockholm', '59.33', '18.07', 'Europe/Stockholm'),
          ('Taipei', '25.03', '121.56', 'Asia/Taipei'),
          ('Tokyo', '35.68', '139.64', 'Asia/Tokyo'),
          ('Tucuman', '-26.81', '-65.22', 'America/Argentina/Tucuman'),
          ('Vienna', '48.21', '16.38', 'Europe/Vienna'),
          ('Warsaw', '52.23', '21.01', 'Europe/Warsaw')]


def getWeatherConditions(lat, lon):
    """Get weather data from FCC weather API.

    Args:
        lat (string): Latitude
        lon (string): Longitude
    """
    try:
        url = 'https://fcc-weather-api.glitch.me/api/current'
        # encode query string for request
        query_strings = {'lon': lon, 'lat': lat}
        # headers to disable caching (in theory)
        headers = {'Pragma': 'no-cache', 'Cache-Control': 'no-cache'}
        while True:
            # get weather data from Free Code Camp Weather API
            r = requests.get(url,
                             params=query_strings,
                             headers=headers,
                             timeout=30)
            data = r.json()
            status_code = r.status_code
            r.close()
            # If data is unavailble city will equal Shuzenji
            if data['name'] != 'Shuzenji':
                break
            print('data unavailable...')
            sleep(3)
        # return data formatted to JSON
        return data, status_code
    except requests.exceptions.Timeout:
        return "Timeout", 408
    except requests.exceptions.ConnectionError:
        return "Connection Error", 404
    except Exception:
        e = sys.exc_info()[0]
        print("Error: {0}".format(e))
        return "Undefined Error", 0


if __name__ == "__main__":
    while 1:
        for city in cities:
            data, code = getWeatherConditions(city[1], city[2])
            if code != 200:
                # Status Code Error
                print (data)
                if not isinstance(data, str):
                    data = "Unknown"
                lcd.clear()
                lcd.message("ERROR {0}\n{1}".format(code, data))
                sleep(4)
            elif not isinstance(data, dict) or "error" in data:
                # Feed Error
                print (data)
                lcd.clear()
                lcd.message("FEED\nERROR")
                sleep(4)
            else:
                # Retrieve city name, condition and temperature
                cond = data['weather'][0]['description']
                temp = data['main']['temp']
                lcd.clear()
                # Display city temperature and condition
                lcd.message(city[0] + ' {0:.1f}'.format(temp))
                lcd.write8(223, True)
                lcd.message("C\n" + cond)
                sleep(3)

                # Display city humidity and local city time
                humidity = data['main']['humidity']
                lcd.clear()
                lcd.message(city[0] + ' {0:.0f}'.format(humidity))
                lcd.message("%\n")
                lcd.message(datetime.now(timezone(city[3])).strftime(
                    '%I:%M %p %z'))
                sleep(3)

                # Poll DHT22
                h, t = dht.read_retry(dht.DHT22, 4)
                lcd.clear()
                # Display time
                lcd.message(datetime.now().strftime('%I:%M %p %b %d'))
                # Display temperature an humidity
                lcd.message("\n{0:0.1f}".format(t))
                lcd.write8(223, True)
                lcd.message("C   {0:0.1f}%".format(h))
                sleep(3)
